@@ -21,7 +21,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: cr_chkpt_req.c,v 1.264.6.2 2009/03/06 19:36:11 phargrov Exp $
+ * $Id: cr_chkpt_req.c,v 1.264.6.4 2011/08/23 21:41:49 phargrov Exp $
  */
 
 #include "cr_module.h"
@@ -321,9 +321,13 @@ static int cr_is_ptrace_parent(struct task_struct *task)
 #elif HAVE_TASK_PTRACE
 	struct task_struct *child;
 	CR_DO_EACH_CHILD(child, task) {
-		if (child->ptrace & PT_PTRACED) {
+	    struct task_struct *thread = child;
+	    if (!thread_group_leader(child)) continue;
+	    do {
+		if (thread->ptrace & PT_PTRACED) {
 			return 1;
 		}
+	    } while_each_thread(child, thread);
 	} CR_WHILE_EACH_CHILD(child, task);
 	return 0;
 #else
@@ -573,23 +577,14 @@ static int add_proc(cr_chkpt_req_t *req, struct task_struct *proc)
 
 	if (!proc->mm || !proc->signal) {
 		result = -EPERM;	/* tried to checkpoint a kernel thread or something ?? */
-       	} else if (atomic_read(&proc->signal->count) == atomic_read(&proc->mm->mm_users)) {
-		/* NEW (nptl) pthreads (or single threaded) */
+	} else {
 		CR_DO_EACH_TASK_TGID(proc->tgid, task) {
 			result = add_task(req, task);
 			if (result) {
 				break;
 			}
 		} CR_WHILE_EACH_TASK_TGID(proc->tgid, task);
-	} else {
-		/* OLD (linuthreads) pthreads */
-		CR_DO_EACH_TASK_PROC(proc, task) {
-			result = add_task(req, task);
-			if (result) {
-				break;
-			}
-		} CR_WHILE_EACH_TASK_PROC(proc, task);
-	}
+	} 
 
 out:
 	return result;
@@ -730,12 +725,15 @@ static int build_req_tree(cr_chkpt_req_t *req, pid_t target)
 
 			/* Add all the children - we get their children in the list_for_each() */
 			CR_DO_EACH_CHILD(child, task) {
-
-				if (child->mm == root_mm) continue;	/* Already visited */
-				result = add_task(req, child);
+			    struct task_struct *thread = child;
+			    if (!thread_group_leader(child)) continue;
+			    do {
+				if (thread->mm == root_mm) continue;	/* Already visited */
+				result = add_task(req, thread);
 				if (result) {
 					goto out_fail;
 				}
+			    } while_each_thread(child, thread);
 			} CR_WHILE_EACH_CHILD(child, task);
 		}
 	}
