@@ -21,7 +21,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: cr_io.c,v 1.77.4.2 2009/03/26 04:22:43 phargrov Exp $
+ * $Id: cr_io.c,v 1.77.4.3 2009/06/26 18:19:51 phargrov Exp $
  */
 
 /*
@@ -896,7 +896,7 @@ struct file *
 cr_dentry_open(struct dentry *dentry, struct vfsmount *mnt, int flags)
 {
     struct file *filp;
-    int acc_mask = ("\000\004\002\006"[(flags)&O_ACCMODE]); /* ICK (from linux/fs/namei.c) */
+    int acc_mask = ("\004\002\006\006"[flags&O_ACCMODE]); /* ACC_MODE(open_to_namei_flags()) */
     int err;
 
     err = cr_permission(dentry->d_inode, acc_mask, NULL);
