@@ -21,7 +21,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: cr_module.h,v 1.285.4.2 2009/06/07 03:27:36 phargrov Exp $
+ * $Id: cr_module.h,v 1.285.4.4 2010/08/12 21:58:32 phargrov Exp $
  */
 
 #ifndef _CR_MODULE_H
@@ -31,7 +31,9 @@
 #include "blcr_config.h"	// Configuration
 
 #include <linux/module.h>
+#ifdef CR_NEED_AUTOCONF_H
 #include <linux/autoconf.h>
+#endif
 
 #include <linux/kernel.h>
 #include <linux/types.h>
@@ -266,8 +268,8 @@ typedef struct cr_rstrt_req_s {
         cr_location_t           src;
 	atomic_t                ref_count;
 	struct task_struct     *cr_restart_task;
-	struct file            *cr_restart_stdin;
-	struct file            *cr_restart_stdout;
+	struct file            *dpipe_in;
+	struct file            *dpipe_out;
 	int	                need_procs;
 	cr_rstrt_state_t        state;
 	wait_queue_head_t	wait;		// place for requester to wait for state changes
