@@ -17,13 +17,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: vmadump_common.c,v 1.86 2008/12/17 03:29:10 phargrov Exp $
+ * $Id: vmadump_common.c,v 1.86.4.8 2011/10/04 21:07:22 phargrov Exp $
  *
  * THIS VERSION MODIFIED FOR BLCR <http://ftg.lbl.gov/checkpoint>
  *-----------------------------------------------------------------------*/
 #include <linux/kernel.h>
 #include <linux/module.h>
 #include <linux/version.h>
+#include "blcr_config.h"
 
 #include <linux/sched.h>
 
@@ -32,14 +33,13 @@
 #include <linux/mm.h>
 #include <linux/mman.h>
 #include <linux/binfmts.h>
-#include <linux/smp_lock.h>
+#if !HAVE_FILE_F_LOCK
+  #include <linux/smp_lock.h>
+#endif
 #include <linux/unistd.h>
 #include <linux/personality.h>
 #include <linux/highmem.h>
 #include <linux/ptrace.h>
-#ifdef HAVE_LINUX_SYSCALLS_H
-  #include <linux/syscalls.h>	/* for mprotect, etc. */
-#endif
 #include <linux/init.h>
 #include <linux/prctl.h>
 #include <asm/pgtable.h>
@@ -549,6 +549,7 @@ directio_start(struct file *filp)
     /*
      * This was basically ripped out of setfl() in 2.6.22, with some #if's
      * added based on 2.6.1->2 and 2.6.7->8 changes to that function.
+     * And for 2.6.29->30 changes (file.f_lock)
      *
      * The alternative is to call sys_fcntl() directly.  This is a good idea.
      * But right now I'm not in the mood to walk the fd array to identify the
@@ -579,6 +580,11 @@ directio_start(struct file *filp)
     /* OK */
   #endif
 
+  #if HAVE_FILE_F_LOCK
+    spin_lock(&filp->f_lock);
+    filp->f_flags |= O_DIRECT;
+    spin_unlock(&filp->f_lock);
+  #else
     /* The lock_kernel() does appear necessary here because both
      * setfl() and ioctl() use it when modifying f_flags.  So, to
      * prevent corruption of f_flags, we take the lock too.
@@ -588,6 +594,7 @@ directio_start(struct file *filp)
     lock_kernel();
     filp->f_flags |= O_DIRECT;
     unlock_kernel();
+  #endif
 
 out:
     return saved_flags;
@@ -596,9 +603,15 @@ out:
 static
 void directio_stop(struct file *filp, unsigned int saved_flags)
 {
+  #if HAVE_FILE_F_LOCK
+    spin_lock(&filp->f_lock);
+    filp->f_flags = saved_flags;
+    spin_unlock(&filp->f_lock);
+  #else
     lock_kernel();
     filp->f_flags = saved_flags;
     unlock_kernel();
+  #endif
 }
 
 /* sys_prctl "by value" */
@@ -628,6 +641,45 @@ int vmadump_sigaltstack_k(const stack_t *ss, stack_t *oss) {
 /*--------------------------------------------------------------------
  *  Process "thawing" routines.
  *------------------------------------------------------------------*/
+long vmad_remap(cr_rstrt_proc_req_t *ctx, unsigned long from_addr,
+                unsigned long to_addr, long len) {
+    long r;
+    unsigned long new_addr;
+    long diff = (from_addr > to_addr) ? (from_addr - to_addr)
+                                      : (to_addr - from_addr);
+    if (diff < len) { /* Check for overlap case */
+        unsigned long tmp_addr;
+        /*CR_WARN_CTX(ctx, "vmad_remap using temporary");*/
+
+        tmp_addr = do_mmap(NULL, 0, len, PROT_NONE, MAP_PRIVATE|MAP_ANONYMOUS, 0);
+        r = (tmp_addr & (PAGE_SIZE-1));
+        if  (r) {
+            CR_ERR_CTX(ctx, "vmad_remap failed to allocate temporary %d", (int)r);
+            goto err;
+        }
+
+        new_addr = sys_mremap(from_addr, len, len, MREMAP_FIXED|MREMAP_MAYMOVE, tmp_addr);
+        if  (new_addr != tmp_addr) {
+            r = (new_addr & (PAGE_SIZE-1)) ? new_addr : -ENOMEM;
+            CR_ERR_CTX(ctx, "vmad_remap failed to use temporary %d", (int)r);
+            goto err;
+        }
+
+        from_addr = tmp_addr;
+    }
+
+    new_addr = sys_mremap(from_addr, len, len, MREMAP_FIXED|MREMAP_MAYMOVE, to_addr);
+    if (new_addr != to_addr) {
+        r = (new_addr & (PAGE_SIZE-1)) ? new_addr : -ENOMEM;
+        CR_ERR_CTX(ctx, "vmad_remap failed %d", (int)r);
+        goto err;
+    }
+
+    r = 0;
+err:
+    return r;
+}
+
 static
 int mmap_file(cr_rstrt_proc_req_t *ctx,
 	      const struct vmadump_vma_header *head, char *filename,
@@ -851,7 +903,13 @@ int load_map(cr_rstrt_proc_req_t *ctx,
 	if (addr != head->start) {
 	    CR_ERR_CTX(ctx, "do_mmap(0, %08lx, %08lx, ...) = 0x%08lx (failed)",
 		   head->start, head->end - head->start, addr);
-	    return -EINVAL;
+            if ((addr != head->start) && IS_ERR((void *) addr)) {
+                r = PTR_ERR((void *) addr);
+            } else {
+                r = -EINVAL;
+            }
+
+	    return r;
 	}
     }
 
@@ -919,6 +977,7 @@ int vmadump_load_sigpending(cr_rstrt_proc_req_t *ctx, struct file *file,
 enum vmad_prctl_type {
    vmad_prctl_int_ref,  /* integer read by reference, written by value*/
    vmad_prctl_int_val,  /* integer read and written by value */
+   vmad_prctl_bool,     /* integer read and written by value, write must be 0 or 1 */
    vmad_prctl_comm,     /* string of length TASK_COMM_LEN, set by reference */
 };
 
@@ -936,6 +995,7 @@ int vmadump_load_prctl(cr_rstrt_proc_req_t *ctx, struct file *file) {
     if (r != sizeof(header)) goto bad_read;
     while (header.option) { /* option==0 is end of the list */
         switch (header.type) {
+        case vmad_prctl_bool:
         case vmad_prctl_int_val:
         case vmad_prctl_int_ref: {
             unsigned int value;
@@ -1089,7 +1149,7 @@ long vmadump_thaw_proc(cr_rstrt_proc_req_t *ctx,
 
     r = vmadump_load_sigpending(ctx, file, &current->pending, 0);
     if (r < 0) goto bad_read;
-    if ((atomic_read(&current->signal->count) == 1) || !(flags & VMAD_DUMP_REGSONLY)) {
+    if (!(flags & VMAD_DUMP_REGSONLY)) {
 	/* Restore shared queue if not actually shared, or if we are the "leader" */
 	r = vmadump_load_sigpending(ctx, file, &current->signal->shared_pending, 1);
 	if (r < 0) goto bad_read;
@@ -1833,13 +1893,13 @@ int vmadump_store_prctl(cr_chkpt_proc_req_t *ctx, struct file *file) {
         {vmad_prctl_int_ref, PR_GET_PDEATHSIG, PR_SET_PDEATHSIG},
     #endif
     #if defined(PR_GET_DUMPABLE)
-        {vmad_prctl_int_val, PR_GET_DUMPABLE, PR_SET_DUMPABLE},
+        {vmad_prctl_bool, PR_GET_DUMPABLE, PR_SET_DUMPABLE},
     #endif
     #if defined(PR_GET_UNALIGN) && defined(GET_UNALIGN_CTL)
         {vmad_prctl_int_ref, PR_GET_UNALIGN, PR_SET_UNALIGN},
     #endif
     #if defined(PR_GET_KEEPCAPS)
-        {vmad_prctl_int_val, PR_GET_KEEPCAPS, PR_SET_KEEPCAPS},
+        {vmad_prctl_bool, PR_GET_KEEPCAPS, PR_SET_KEEPCAPS},
     #endif
     #if defined(PR_GET_FPEMU) && defined(GET_FPEMU_CTL)
         {vmad_prctl_int_ref, PR_GET_FPEMU, PR_SET_FPEMU},
@@ -1898,7 +1958,15 @@ int vmadump_store_prctl(cr_chkpt_proc_req_t *ctx, struct file *file) {
         const int get = vmad_prctl_tbl[i].get_option;
         const int set = vmad_prctl_tbl[i].set_option;
 
-        switch (vmad_prctl_tbl[i].type) {
+        switch (type) {
+        case vmad_prctl_bool:
+            r = vmadump_prctl_v(get, 0);
+            if (!IS_ERR((void *)r)) { /* ICK */
+                value = r ? 1 : 0;
+                r = 0;
+            }
+            break;
+
         case vmad_prctl_int_val:
             r = vmadump_prctl_v(get, 0);
             if (!IS_ERR((void *)r)) { /* ICK */
@@ -1996,7 +2064,7 @@ loff_t vmadump_freeze_proc(cr_chkpt_proc_req_t *ctx, struct file *file,
     r = vmadump_store_sigpending(ctx, file, &current->pending);
     if (r < 0) goto err;
     bytes += r;
-    if ((atomic_read(&current->signal->count) == 1) || !(flags & VMAD_DUMP_REGSONLY)) {
+    if (!(flags & VMAD_DUMP_REGSONLY)) {
 	/* Dump shared queue if not actually shared, or if we are the "leader" */
 	r = vmadump_store_sigpending(ctx, file, &current->signal->shared_pending);
 	if (r < 0) goto err;
